/*! \file
 *  \brief Тестирование функции dotprod
 *  \copyright 2018 RnD Center ELVEES, JSC
 *  \author Фролов Андрей
 *  \date 2018.04.12
 */
#include "tests.h"

int main()
{
    int failed_count = 0;
    int size[TEST_COUNT] = {256, 512, 1024, 2048, 8192};
    int print = 0;

    print_table_header();

    #ifndef LOCAL_MEM
        void* src0_pos = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
        void* src1_pos = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
        void* src0_neg = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
        void* src1_neg = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
        void* src0_rnd = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
        void* src1_rnd = memalign(64, size[TEST_COUNT - 1] * sizeof(int64_t));
    #else
        #ifdef BARE_METAL
        void* src0_pos = &__local_mem;
        #else
        uint32_t l2dc_val = *L2DC;
        FLUSH_ALL_CACHES();
        *L2DC &= 0xFFFFFFBF;
        void* src0_pos = &xyram_data;
        #endif
        void* src1_pos = src0_pos + size[TEST_COUNT - 1] * sizeof(int64_t);
        void* src0_neg = src1_pos + size[TEST_COUNT - 1] * sizeof(int64_t);
        void* src1_neg = src0_neg + size[TEST_COUNT - 1] * sizeof(int64_t);
        void* src0_rnd = src1_neg + size[TEST_COUNT - 1] * sizeof(int64_t);
        void* src1_rnd = src0_rnd + size[TEST_COUNT - 1] * sizeof(int64_t);
    #endif

    create_vector_s16((int16_t*)src0_pos, size[TEST_COUNT - 1], 1);
    create_vector_s16((int16_t*)src1_pos, size[TEST_COUNT - 1], 1);
    create_vector_s16((int16_t*)src0_neg, size[TEST_COUNT - 1], -1);
    create_vector_s16((int16_t*)src1_neg, size[TEST_COUNT - 1], -1);
    create_vector_s16((int16_t*)src0_rnd, size[TEST_COUNT - 1], 0);
    create_vector_s16((int16_t*)src1_rnd, size[TEST_COUNT - 1], 0);
    for(int i = 0; i < TEST_COUNT; ++i)
    {
        int32_t input_bytes = size[i] * sizeof(int16_t) * 2;
        int32_t ti_tics = size[i] / 4 + 31;

        #ifndef DISABLE_NEG_TEST_DATA

        printf("| dotprod_short_neg      | %14d |", size[i]);
        failed_count += test_dotprod((int16_t*)src0_neg, (int16_t*)src1_neg, size[i], print, input_bytes, ti_tics);
        #endif

        printf("| dotprod_short_rnd      | %14d |", size[i]);
        failed_count += test_dotprod((int16_t*)src0_rnd, (int16_t*)src1_rnd, size[i], print, input_bytes, ti_tics);

        #ifndef DISABLE_POS_TEST_DATA
        printf("| dotprod_short_pos      | %14d |", size[i]);
        failed_count += test_dotprod((int16_t*)src0_pos, (int16_t*)src1_pos, size[i], print, input_bytes, ti_tics);
        #endif
    }

    create_vector_s32((int32_t*)src0_pos, size[TEST_COUNT - 1], 1);
    create_vector_s32((int32_t*)src1_pos, size[TEST_COUNT - 1], 1);
    create_vector_s32((int32_t*)src0_neg, size[TEST_COUNT - 1], -1);
    create_vector_s32((int32_t*)src1_neg, size[TEST_COUNT - 1], -1);
    create_vector_s32((int32_t*)src0_rnd, size[TEST_COUNT - 1], 0);
    create_vector_s32((int32_t*)src1_rnd, size[TEST_COUNT - 1], 0);
    for(int i = 0; i < TEST_COUNT; ++i)
    {
        int32_t input_bytes = size[i] * sizeof(int32_t) * 2;
        int32_t ti_tics = 0;

        #ifndef DISABLE_NEG_TEST_DATA
        printf("| dotprod_int_neg        | %14d |", size[i]);
        failed_count += test_dotprod32((int32_t*)src0_neg, (int32_t*)src1_neg, size[i], print, input_bytes, ti_tics);
        #endif

        printf("| dotprod_int_rnd        | %14d |", size[i]);
        failed_count += test_dotprod32((int32_t*)src0_rnd, (int32_t*)src1_rnd, size[i], print, input_bytes, ti_tics);

        #ifndef DISABLE_POS_TEST_DATA
        printf("| dotprod_int_pos        | %14d |", size[i]);
        failed_count += test_dotprod32((int32_t*)src0_pos, (int32_t*)src1_pos, size[i], print, input_bytes, ti_tics);
        #endif
    }

    create_vector_float((float*)src0_pos, size[TEST_COUNT - 1], 1);
    create_vector_float((float*)src1_pos, size[TEST_COUNT - 1], 1);
    create_vector_float((float*)src0_neg, size[TEST_COUNT - 1], -1);
    create_vector_float((float*)src1_neg, size[TEST_COUNT - 1], -1);
    create_vector_float((float*)src0_rnd, size[TEST_COUNT - 1], 0);
    create_vector_float((float*)src1_rnd, size[TEST_COUNT - 1], 0);
    for(int i = 0; i < TEST_COUNT; ++i)
    {
        int32_t input_bytes = size[i] * sizeof(float) * 2;
        int32_t ti_tics = 0;

        #ifndef DISABLE_NEG_TEST_DATA
        printf("| dotprod_fl_neg         | %14d |", size[i]);
        failed_count += test_dotprod_fl((float*)src0_neg, (float*) src1_neg, size[i], print, input_bytes, ti_tics);
        #endif

        printf("| dotprod_fl_rnd         | %14d |", size[i]);
        failed_count += test_dotprod_fl((float*)src0_rnd, (float*) src1_rnd, size[i], print, input_bytes, ti_tics);

        #ifndef DISABLE_POS_TEST_DATA
        printf("| dotprod_fl_pos         | %14d |", size[i]);
        failed_count += test_dotprod_fl((float*)src0_pos, (float*) src1_pos, size[i], print, input_bytes, ti_tics);
        #endif
    }

    create_vector_double((double*)src0_pos, size[TEST_COUNT - 1], 1);
    create_vector_double((double*)src1_pos, size[TEST_COUNT - 1], 1);
    create_vector_double((double*)src0_neg, size[TEST_COUNT - 1], -1);
    create_vector_double((double*)src1_neg, size[TEST_COUNT - 1], -1);
    create_vector_double((double*)src0_rnd, size[TEST_COUNT - 1], 0);
    create_vector_double((double*)src1_rnd, size[TEST_COUNT - 1], 0);
    for(int i = 0; i < TEST_COUNT; ++i)
    {
        int32_t input_bytes = size[i] * sizeof(double) * 2;
        int32_t ti_tics = 0;

        #ifndef DISABLE_NEG_TEST_DATA
        printf("| dotprod_db_neg         | %14d |", size[i]);
        failed_count += test_dotprod_db((double*)src0_neg, (double*)src1_neg, size[i], print, input_bytes, ti_tics);
        #endif

        printf("| dotprod_db_rnd         | %14d |", size[i]);
        failed_count += test_dotprod_db((double*)src0_rnd, (double*)src1_rnd, size[i], print, input_bytes, ti_tics);

        #ifndef DISABLE_POS_TEST_DATA
        printf("| dotprod_db_pos         | %14d |", size[i]);
        failed_count += test_dotprod_db((double*)src0_pos, (double*)src1_pos, size[i], print, input_bytes, ti_tics);
        #endif
    }

    #ifndef LOCAL_MEM
        free(src0_pos);
        free(src1_pos);
        free(src0_neg);
        free(src1_neg);
        free(src0_rnd);
        free(src1_rnd);
    #else
    #ifndef BARE_METAL
        *L2DC = l2dc_val;
    #endif
    #endif

    return failed_count;
}

int test_dotprod(int16_t* src0, int16_t* src1, int size, int print, int32_t input_bytes, int32_t ti_tics)
{
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    int32_t res = ref_dotprod(src0, src1, size);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    int32_t c = dotprod(src0, src1, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s16(src0, size);
        printf("vect2:");
        print_vector_s16(src1, size);
        printf("dsp_res:");
        printf("%d\n", c);
        printf("ref_res:");
        printf("%d\n", res);
    }

    if (c == res)
    {
        ret = 0;
    }
    else
    {
        ret = 1;
    }

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, ti_tics);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_dotprod32(int32_t* src0, int32_t* src1, int size, int print, int32_t input_bytes, int32_t ti_tics)
{
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    int64_t res = ref_dotprod32(src0, src1, size);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    int64_t c = dotprod32(src0, src1, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_s32(src0, size);
        printf("vect2:");
        print_vector_s32(src1, size);
        printf("dsp_res:");
        printf("%lld\n", c);
        printf("ref_res:");
        printf("%lld\n", res);
    }

    if (c == res)
    {
        ret = 0;
    }
    else
    {
        ret = 1;
    }

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, ti_tics);
    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_dotprod_fl(float* src0, float* src1, int size, int print, int32_t input_bytes, int32_t ti_tics)
{
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    float res = ref_dotprod_fl(src0, src1, size);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    float c = dotprod_fl(src0, src1, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_float(src0, size);
        printf("vect2:");
        print_vector_float(src1, size);
        printf("dsp_res:");
        printf("%f\n", c);
        printf("ref_res:");
        printf("%f\n", res);
    }

#ifdef EPS
    ret = compare_float_eps(&c, &res, 1, EPS);
#else
    ret = compare_float_eps(&c, &res, 1, 0.001);
#endif

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, ti_tics);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}

int test_dotprod_db(double* src0, double* src1, int size, int print, int32_t input_bytes, int32_t ti_tics)
{
    int ret = 0;

    uint32_t tic_count[2], instruction_count[2];
    uint32_t ref_tic_count[2], ref_instruction_count[2];

    count_tics(ref_tic_count, ref_instruction_count);
    double res = ref_dotprod_db(src0, src1, size);
    count_tics(&ref_tic_count[1], &ref_instruction_count[1]);

    count_tics(tic_count, instruction_count);
    double c = dotprod_db(src0, src1, size);
    count_tics(&tic_count[1], &instruction_count[1]);

    if (print)
    {
        printf("vect1:");
        print_vector_double(src0, size);
        printf("vect2:");
        print_vector_double(src1, size);
        printf("dsp_res:");
        printf("%f\n", c);
        printf("ref_res:");
        printf("%f\n", res);
    }

    ret = compare_double_eps(&c, &res, 1, 0.00001);

    print_performance(ref_tic_count, ref_instruction_count,
                      tic_count, instruction_count,
                      input_bytes, ti_tics);

    if (ret == 0)
        printf(" passed |\n");
    else
        printf(" failed |\n");

    return ret;
}
